@@ -7,6 +7,7 @@ import logging
 from argparse import ArgumentParser
 
 from . import utils
+from .inirama import Namespace
 
 
 default_linters = 'pep8', 'pyflakes', 'mccabe'
@@ -18,33 +19,34 @@ logger.addHandler(stream)
 SKIP_PATTERN = '# nolint'
 
 
-def run(path, ignore=None, select=None, linters=default_linters, **meta):  # nolint
+def run(path, ignore=None, select=None, linters=default_linters, config=None,
+        **meta):
     errors = []
     ignore = ignore and list(ignore) or []
     select = select and list(select) or []
 
-    for lint in linters:
-        try:
-            linter = getattr(utils, lint)
-        except AttributeError:
-            logging.warning("Linter `{0}` not found.".format(lint))
-            continue
+    try:
+        with open(path, 'rU') as f:
+            code = f.read() + '\n\n'
+            params = config or parse_modeline(code)
+            params['skip'] = [False]
+            for line in code.split('\n'):
+                params['skip'].append(line.endswith(SKIP_PATTERN))
 
-        try:
-            with open(path, "rU") as f:
-                code = f.read() + '\n\n'
-                params = parse_modeline(code)
-                params['skip'] = [False]
-                for line in code.split('\n'):
-                    params['skip'].append(line.endswith(SKIP_PATTERN))
+            if params.get('lint_ignore'):
+                ignore += params.get('lint_ignore').split(',')
 
-                if params.get('lint_ignore'):
-                    ignore += params.get('lint_ignore').split(',')
+            if params.get('lint_select'):
+                select += params.get('lint_select').split(',')
 
-                if params.get('lint_select'):
-                    select += params.get('lint_select').split(',')
+            if params.get('lint'):
+                for lint in linters:
+                    try:
+                        linter = getattr(utils, lint)
+                    except AttributeError:
+                        logging.warning("Linter `{0}` not found.".format(lint))
+                        continue
 
-                if params.get('lint'):
                     result = linter(path, code=code, **meta)
                     for e in result:
                         e['col'] = e.get('col') or 0
@@ -57,27 +59,27 @@ def run(path, ignore=None, select=None, linters=default_linters, **meta):  # nol
                         if not params['skip'][e['lnum']]:
                             errors.append(e)
 
-        except IOError as e:
-            errors.append(dict(
-                lnum=0,
-                type='E',
-                col=0,
-                text=str(e)
-            ))
-        except SyntaxError as e:
-            errors.append(dict(
-                lnum=e.lineno or 0,
-                type='E',
-                col=e.offset or 0,
-                text=e.args[0]
-            ))
-            break
+    except IOError as e:
+        errors.append(dict(
+            lnum=0,
+            type='E',
+            col=0,
+            text=str(e)
+        ))
 
-        except Exception as e:
-            import traceback
-            logging.error(traceback.format_exc())
+    except SyntaxError as e:
+        errors.append(dict(
+            lnum=e.lineno or 0,
+            type='E',
+            col=e.offset or 0,
+            text=e.args[0]
+        ))
 
-    errors = [e for e in errors if _ignore_error(e, select, ignore)]
+    except Exception:
+        import traceback
+        logging.error(traceback.format_exc())
+
+    errors = [er for er in errors if _ignore_error(er, select, ignore)]
     return sorted(errors, key=lambda x: x['lnum'])
 
 
@@ -92,8 +94,9 @@ def _ignore_error(e, select, ignore):
 
 
 def shell():
+    curdir = getcwd()
     parser = ArgumentParser(description="Code audit tool for python.")
-    parser.add_argument("path", nargs='?', default=getcwd(),
+    parser.add_argument("path", nargs='?', default=curdir,
                         help="Path on file or directory.")
     parser.add_argument(
         "--verbose", "-v", action='store_true', help="Verbose mode.")
@@ -125,50 +128,70 @@ def shell():
     parser.add_argument("--report", "-r", help="Filename for report.")
     parser.add_argument("--hook", action="store_true",
                         help="Install Git (Mercurial) hook.")
-    args = parser.parse_args()
+    parser.add_argument(
+        "--options", "-o", default=op.join(curdir, 'pylama.ini'),
+        help="Select configuration file. By default is '<CURDIR>/pylama.ini'")
+    options = parser.parse_args()
+    actions = dict((a.dest, a) for a in parser._actions)
+
+    config = Namespace()
+    config.silent_read = False
+    config.default_section = 'main'
+    config.read(options.options)
+    for k, v in config.default.items():
+        action = actions.get(k)
+        if action:
+            setattr(options, action.dest, action.type(v))
 
     # Setup logger
-    logger.setLevel(logging.INFO if args.verbose else logging.WARN)
-    if args.report:
+    logger.setLevel(logging.INFO if options.verbose else logging.WARN)
+    if options.report:
         logger.removeHandler(stream)
-        logger.addHandler(logging.FileHandler(args.report, mode='w'))
+        logger.addHandler(logging.FileHandler(options.report, mode='w'))
 
-    if args.hook:
+    if options.hook:
         from .hook import install_hook
-        return install_hook(args.path)
+        return install_hook(options.path)
 
-    paths = [args.path]
+    paths = [options.path]
 
-    if op.isdir(args.path):
+    if op.isdir(options.path):
         paths = []
-        for root, _, files in walk(args.path):
+        for root, _, files in walk(options.path):
             paths += [op.join(root, f) for f in files if f.endswith('.py')]
 
     check_files(
         paths,
-        rootpath=args.path,
-        skip=args.skip,
-        frmt=args.format,
-        ignore=args.ignore,
-        select=args.select,
-        linters=args.linters,
-        complexity=args.complexity,
+        rootpath=options.path,
+        skip=options.skip,
+        frmt=options.format,
+        ignore=options.ignore,
+        select=options.select,
+        linters=options.linters,
+        complexity=options.complexity,
+        config=config,
     )
 
 
 def check_files(paths, rootpath=None, skip=None, frmt="pep8",
                 select=None, ignore=None, linters=default_linters,
-                complexity=default_complexity):
+                complexity=default_complexity, config=None):
     rootpath = rootpath or getcwd()
     pattern = "%(rel)s:%(lnum)s:%(col)s: %(text)s"
     if frmt == 'pylint':
         pattern = "%(rel)s:%(lnum)s: [%(type)s] %(text)s"
 
+    params = dict()
+    if config:
+        for key, section in config.sections.items():
+            if key != 'main':
+                params[op.abspath(key)] = prepare_params(section)
+
     errors = []
     for path in skip_paths(skip, paths):
         logger.info("Parse file: %s" % path)
-        errors = run(path, ignore=ignore, select=select,
-                     linters=linters, complexity=complexity)
+        errors = run(path, ignore=ignore, select=select, linters=linters,
+                     complexity=complexity, config=params.get(path))
         for error in errors:
             try:
                 error['rel'] = op.relpath(
@@ -201,5 +224,11 @@ def parse_modeline(code):
     return params
 
 
+def prepare_params(section):
+    params = dict(section)
+    params['lint'] = int(params.get('lint', 1))
+    return params
+
+
 if __name__ == '__main__':
     shell()
